#!/usr/bin/env ruby

require 'rubygems' if RUBY_VERSION < '1.9.0'

require 'sensu-plugin/check/cli'
require 'open-uri'
require 'fileutils'
require 'zip/zip'

class UpdatePlugin < Sensu::Plugin::Check::CLI
    SENSU_DIR = '/etc/sensu'
    SENSU_TMP_DIR = SENSU_DIR + '/tmp'
    SENSU_PLUGIN_DIR = SENSU_DIR + '/plugins'
    UPDATE_SERVER = [
        'http://monitoring-pangyo.cloud.toast.com:11000'
    ]
    
    def run
        begin
            FileUtils.rm_rf(SENSU_TMP_DIR)
            FileUtils.mkdir_p(SENSU_TMP_DIR)

            getFileFromServer("VERSION")

            if FileUtils.compare_file(SENSU_TMP_DIR + '/VERSION', SENSU_PLUGIN_DIR + '/VERSION')
                FileUtils.rm_rf(SENSU_TMP_DIR)
                ok "there is no available update."
            else
                getFileFromServer("plugins.zip")

                Zip::ZipFile.open(SENSU_TMP_DIR + "/plugins.zip") do |zipFile|
                    zipFile.each do |file|
                        zipFile.extract(file, File.join(SENSU_PLUGIN_DIR, file.name)){true}
                    end
                end

                FileUtils.cp_r(SENSU_TMP_DIR + "/VERSION", SENSU_PLUGIN_DIR)
                FileUtils.rm_rf(SENSU_TMP_DIR)

                ok "update success"
            end
        rescue
            critical "update fail"
        end
    end

    def getFileFromServer(fileName)
        UPDATE_SERVER.shuffle!
        UPDATE_SERVER.each { |server|
            begin
                File.open(SENSU_TMP_DIR + "/" + fileName, "w") do |saved_file|
                    open(server + "/vm/latest/" + fileName, "r") do |read_file|
                        saved_file.write(read_file.read)
                    end
                end
                break
            rescue
            end
        }
    end
end
